/*....This program scales (amplifies) and smooths 8-bit digitized time 
 .....histories.  This improves the clarity of digitized sound.
 .....
 .....I give this program to the public domain.  If you modify it, or
 .....have suggestions or comments, please drop me a note at
 .....Comp-U-Ease, 408-286-8332, or The Engineers' Club, 408-265-3353.
 .....
 .....          Usage:  dig infile outfile
 .....                                                    --Bill Weitze
 .....
 .....04-16-91, Rev. 1.1:  Now reads input into a vector, speeding things.
 .....06-10-91, Rev. 1.2:  Added smoothing option and other improvements.
 .....09-24-91, Rev. 1.3:  Simplified code, adapted for MS Fortran 5.1.
 .....*/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

main( int argc, char *argv[] )
{
   char unsigned ampc[20000];
   int   amp, iend, ifile, ispike, ave;
   long  itot, j1, j;
   float factor;
   FILE  *fpin, *fpout, *fplist;     /*  binary in/output files, list file  */

   if ( argv[1] == NULL )
   {
      printf("Program format: DIG infile outfile");
      exit(1);
   }

   if( (fpin = fopen( argv[1],"r+b" )) == NULL )
   {
      printf( "error on opening input file ""%s""\n", argv[1] );
      exit(1);
   }

  if( (fpout = fopen( argv[2],"w+b" )) == NULL )
   {
      printf( "error on opening output file ""%s""\n", argv[2] );
      exit(1);
   }

   printf ("Enter volume factor:  ");
   scanf ("%f", &factor);
   printf ("Enter spike limit (0=smoothest, 255=spikiest):  ");
   scanf ("%i", &ispike);
   printf ("Would you like a data listing (type 1 for yes)?\n");
   printf ("(file is 12 times the size of the input file.) ");
   scanf ("%i", &ifile);
   if (ifile == 1)
   {
      if( (fplist = fopen( "list", "w+t" )) == NULL )
      {
         printf( "error on opening list file ""%s""\n", "list" );
         exit(1);
      }
   }
/*
 .....Read file and store data in array
*/
   amp=0;
   itot=0;
   iend=0;
   while (iend != 1)
   {
      printf ("Reading input file at byte %i\n", itot);
      if ( (j1 = fread(ampc, sizeof(char), 20000, fpin)) != 20000 ) iend=1;
      printf( "Read %i bytes, now modifying data...\n", j1 );
      for (j=0; j<=j1-1; j++)
      {
         ave = (amp + (int) ampc[j+1]-128) / 2;
         amp = (int) ampc[j]-128;
         if (j == j1-1) ave = amp;
         ave = min(max( (int) (factor*amp),ave-ispike),ave+ispike);
         ampc[j] = (char) (min(max(ave,-128),127)+128);
      }
      printf("Writing output file ...\n");
      fwrite( ampc, sizeof(char), j1, fpout);
      if (ifile == 1) fprintf( fplist, "%i %i",(j+itot,ampc[j]-128, j=0,j1-1));
      itot=itot+j1;
   }
   fclose( fpin );
   fclose( fpout );
   fclose( fplist );
}
